from django.conf import settings
from django.core import checks

from django_recaptcha.constants import TEST_PRIVATE_KEY, TEST_PUBLIC_KEY


def recaptcha_key_check(app_configs, **kwargs):
    errors = []
    private_key = getattr(settings, "RECAPTCHA_PRIVATE_KEY", TEST_PRIVATE_KEY)
    public_key = getattr(settings, "RECAPTCHA_PUBLIC_KEY", TEST_PUBLIC_KEY)

    if private_key == TEST_PRIVATE_KEY or public_key == TEST_PUBLIC_KEY:
        errors.extend(
            [
                checks.Error(
                    "RECAPTCHA_PRIVATE_KEY or RECAPTCHA_PUBLIC_KEY is making use"
                    " of the Google test keys and will not behave as expected in a"
                    " production environment",
                    hint="Update settings.RECAPTCHA_PRIVATE_KEY"
                    " and/or settings.RECAPTCHA_PUBLIC_KEY. Alternatively this check"
                    " can be ignored by adding"
                    " `SILENCED_SYSTEM_CHECKS = ['django_recaptcha.recaptcha_test_key_error']`"
                    " to your settings file.",
                    id="django_recaptcha.recaptcha_test_key_error",
                )
            ]
        )
    return errors



# Test keys as per https://developers.google.com/recaptcha/docs/faq
# "With the following test keys, you will always get No CAPTCHA and all
# verification requests will pass."
TEST_PUBLIC_KEY = "6LeIxAcTAAAAAJcZVRqyHh71UMIEGNQ_MXjiZKhI"
TEST_PRIVATE_KEY = "6LeIxAcTAAAAAGG-vFI1TnRWxMZNFuojJ4WifJWe"
DEFAULT_RECAPTCHA_DOMAIN = "www.google.com"



DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "test.sqlite",
    }
}

INSTALLED_APPS = [
    "django_recaptcha",
]

RECAPTCHA_PRIVATE_KEY = "privkey"
RECAPTCHA_PUBLIC_KEY = "pubkey"

SECRET_KEY = "SECRET_KEY"



from django.apps import AppConfig
from django.core.checks import Tags, register

from django_recaptcha.checks import recaptcha_key_check


class DjangoRecaptchaConfig(AppConfig):
    name = "django_recaptcha"
    verbose_name = "Django reCAPTCHA"

    def ready(self):
        register(recaptcha_key_check, Tags.security)



import logging
import sys
from urllib.error import HTTPError

from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.utils.translation import gettext_lazy as _

from django_recaptcha import client
from django_recaptcha.constants import TEST_PRIVATE_KEY, TEST_PUBLIC_KEY
from django_recaptcha.widgets import ReCaptchaBase, ReCaptchaV2Checkbox, ReCaptchaV3

logger = logging.getLogger(__name__)


class ReCaptchaField(forms.CharField):
    widget = ReCaptchaV2Checkbox
    default_error_messages = {
        "captcha_invalid": _("Error verifying reCAPTCHA, please try again."),
        "captcha_error": _("Error verifying reCAPTCHA, please try again."),
    }

    def __init__(self, public_key=None, private_key=None, *args, **kwargs):
        """
        ReCaptchaField can accepts attributes which is a dictionary of
        attributes to be passed to the ReCaptcha widget class. The widget will
        loop over any options added and create the RecaptchaOptions
        JavaScript variables as specified in
        https://developers.google.com/recaptcha/docs/display#render_param
        """
        super().__init__(*args, **kwargs)

        if not isinstance(self.widget, ReCaptchaBase):
            raise ImproperlyConfigured(
                "django_recaptcha.fields.ReCaptchaField.widget"
                " must be a subclass of django_recaptcha.widgets.ReCaptchaBase"
            )

        # reCAPTCHA fields are always required.
        self.required = True

        # Setup instance variables.
        self.private_key = private_key or getattr(
            settings, "RECAPTCHA_PRIVATE_KEY", TEST_PRIVATE_KEY
        )
        self.public_key = public_key or getattr(
            settings, "RECAPTCHA_PUBLIC_KEY", TEST_PUBLIC_KEY
        )

        # Update widget attrs with data-sitekey.
        self.widget.attrs["data-sitekey"] = self.public_key

    def get_remote_ip(self):
        f = sys._getframe()
        while f:
            request = f.f_locals.get("request")
            if request:
                remote_ip = request.META.get("REMOTE_ADDR", "")
                forwarded_ip = request.META.get("HTTP_X_FORWARDED_FOR", "")
                ip = remote_ip if not forwarded_ip else forwarded_ip
                return ip
            f = f.f_back

    def validate(self, value):
        super().validate(value)

        try:
            check_captcha = client.submit(
                recaptcha_response=value,
                private_key=self.private_key,
                remoteip=self.get_remote_ip(),
            )

        except HTTPError:  # Catch timeouts, etc
            raise ValidationError(
                self.error_messages["captcha_error"], code="captcha_error"
            )

        if not check_captcha.is_valid:
            logger.warning(
                "ReCAPTCHA validation failed due to: %s" % check_captcha.error_codes
            )
            raise ValidationError(
                self.error_messages["captcha_invalid"], code="captcha_invalid"
            )

        if (
            isinstance(self.widget, ReCaptchaV3)
            and check_captcha.action != self.widget.action
        ):
            logger.warning(
                "ReCAPTCHA validation failed due to: mismatched action. Expected '%s' but received '%s' from captcha server."
                % (self.widget.action, check_captcha.action)
            )
            raise ValidationError(
                self.error_messages["captcha_invalid"], code="captcha_invalid"
            )

        required_score = self.widget.attrs.get("required_score")
        if required_score:
            # Our score values need to be floats, as that is the expected
            # response from the Google endpoint. Rather than ensure that on
            # the widget, we do it on the field to better support user
            # subclassing of the widgets.
            required_score = float(required_score)

            # If a score was expected but non was returned, default to a 0,
            # which is the lowest score that it can return. This is to do our
            # best to assure a failure here, we can not assume that a form
            # that needed the threshold should be valid if we didn't get a
            # value back.
            score = float(check_captcha.extra_data.get("score", 0))

            if required_score > score:
                logger.warning(
                    "ReCAPTCHA validation failed due to its score of %s"
                    " being lower than the required amount." % score
                )
                raise ValidationError(
                    self.error_messages["captcha_invalid"], code="captcha_invalid"
                )



import uuid
from urllib.parse import urlencode

from django.conf import settings
from django.forms import widgets

from django_recaptcha.constants import DEFAULT_RECAPTCHA_DOMAIN


class ReCaptchaBase(widgets.Widget):
    """
    Base widget to be used for Google ReCAPTCHA.

    public_key -- String value: can optionally be passed to not make use of the
        project wide Google Site Key.
    """

    recaptcha_response_name = "g-recaptcha-response"

    def __init__(self, api_params=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.uuid = uuid.uuid4().hex
        self.api_params = api_params or {}

        if not self.attrs.get("class", None):
            self.attrs["class"] = "g-recaptcha"

    def value_from_datadict(self, data, files, name):
        return data.get(self.recaptcha_response_name, None)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        params = urlencode(self.api_params)
        context.update(
            {
                "public_key": self.attrs["data-sitekey"],
                "widget_uuid": self.uuid,
                "api_params": params,
                "recaptcha_domain": getattr(
                    settings, "RECAPTCHA_DOMAIN", DEFAULT_RECAPTCHA_DOMAIN
                ),
            }
        )
        return context

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = super().build_attrs(base_attrs, extra_attrs)
        attrs["data-widget-uuid"] = self.uuid

        # Support the ability to override some of the Google data attrs.
        attrs["data-callback"] = base_attrs.get(
            "data-callback", "onSubmit_%s" % self.uuid
        )
        attrs["data-size"] = base_attrs.get("data-size", "normal")
        return attrs


class ReCaptchaV2Checkbox(ReCaptchaBase):
    template_name = "django_recaptcha/widget_v2_checkbox.html"


class ReCaptchaV2Invisible(ReCaptchaBase):
    template_name = "django_recaptcha/widget_v2_invisible.html"

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = super().build_attrs(base_attrs, extra_attrs)

        # Invisible reCAPTCHA should not have another size
        attrs["data-size"] = "invisible"
        return attrs


class ReCaptchaV3(ReCaptchaBase):
    input_type = "hidden"
    template_name = "django_recaptcha/widget_v3.html"

    def __init__(self, api_params=None, action=None, *args, **kwargs):
        super().__init__(api_params=api_params, *args, **kwargs)
        if not self.attrs.get("required_score", None):
            self.attrs["required_score"] = getattr(
                settings, "RECAPTCHA_REQUIRED_SCORE", None
            )
        self.action = action

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = super().build_attrs(base_attrs, extra_attrs)
        return attrs

    def value_from_datadict(self, data, files, name):
        return data.get(name)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context.update({"action": self.action})
        return context



from importlib import reload

from django.core.checks import Error
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, override_settings

import django_recaptcha


class TestInit(TestCase):
    def test_setting_instance_check(self):
        with override_settings(RECAPTCHA_PROXY="not a dict"):
            with self.assertRaises(ImproperlyConfigured) as error:
                reload(django_recaptcha)
            self.assertEqual(
                error.exception.args, ("Setting RECAPTCHA_PROXY is not of type", dict)
            )
        with override_settings(RECAPTCHA_VERIFY_REQUEST_TIMEOUT="not an int"):
            with self.assertRaises(ImproperlyConfigured) as error:
                reload(django_recaptcha)
            self.assertEqual(
                error.exception.args,
                ("Setting RECAPTCHA_VERIFY_REQUEST_TIMEOUT is not of type", int),
            )
        with override_settings(RECAPTCHA_DOMAIN=1):
            with self.assertRaises(ImproperlyConfigured) as error:
                reload(django_recaptcha)
            self.assertEqual(
                error.exception.args, ("Setting RECAPTCHA_DOMAIN is not of type", str)
            )
        with override_settings(RECAPTCHA_PUBLIC_KEY=1):
            with self.assertRaises(ImproperlyConfigured) as error:
                reload(django_recaptcha)
            self.assertEqual(
                error.exception.args,
                ("Setting RECAPTCHA_PUBLIC_KEY is not of type", str),
            )
        with override_settings(RECAPTCHA_PRIVATE_KEY=1):
            with self.assertRaises(ImproperlyConfigured) as error:
                reload(django_recaptcha)
            self.assertEqual(
                error.exception.args,
                ("Setting RECAPTCHA_PRIVATE_KEY is not of type", str),
            )

    @override_settings(
        RECAPTCHA_PRIVATE_KEY=django_recaptcha.constants.TEST_PRIVATE_KEY
    )
    def test_test_key_check(self):
        check_errors = django_recaptcha.checks.recaptcha_key_check("someconf")
        expected_errors = [
            Error(
                "RECAPTCHA_PRIVATE_KEY or RECAPTCHA_PUBLIC_KEY is making use"
                " of the Google test keys and will not behave as expected in a"
                " production environment",
                hint="Update settings.RECAPTCHA_PRIVATE_KEY"
                " and/or settings.RECAPTCHA_PUBLIC_KEY. Alternatively this"
                " check can be ignored by adding"
                " `SILENCED_SYSTEM_CHECKS ="
                " ['django_recaptcha.recaptcha_test_key_error']`"
                " to your settings file.",
                id="django_recaptcha.recaptcha_test_key_error",
            )
        ]
        self.assertEqual(check_errors, expected_errors)



from setuptools import find_packages, setup

long_desc = open("README.md").read() + "\n\n" + open("AUTHORS.md").read()

setup(
    name="django-recaptcha",
    version="4.0.0",
    description="Django recaptcha form field/widget app.",
    long_description=long_desc,
    long_description_content_type="text/markdown",
    author="Torchbox and individual contributors",
    author_email="hello@torchbox.com",
    license="BSD",
    url="https://github.com/torchbox/django-recaptcha",
    project_urls={
        "Changelog": "https://github.com/torchbox/django-recaptcha/blob/main/CHANGELOG.md",
        "Issue Tracker": "https://github.com/torchbox/django-recaptcha/issues",
        "Discussions": "https://github.com/torchbox/django-recaptcha/discussions",
    },
    packages=find_packages(),
    install_requires=["django"],
    keywords=["django", "reCAPTCHA", "reCAPTCHA v2", "reCAPTCHA v3"],
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    zip_safe=False,
)



import django
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

SETTINGS_TYPES = {
    "RECAPTCHA_DOMAIN": str,
    "RECAPTCHA_PRIVATE_KEY": str,
    "RECAPTCHA_PROXY": dict,
    "RECAPTCHA_PUBLIC_KEY": str,
    "RECAPTCHA_VERIFY_REQUEST_TIMEOUT": int,
}

# Validate settings types.
for variable, instance_type in SETTINGS_TYPES.items():
    if hasattr(settings, variable) and not isinstance(
        getattr(settings, variable), instance_type
    ):
        raise ImproperlyConfigured(
            "Setting %s is not of type" % variable, instance_type
        )

if django.VERSION < (3, 2):
    default_app_config = "django_recaptcha.apps.DjangoRecaptchaConfig"
